@@ -1,8 +1,10 @@
 import datetime
 import sys
+import logging
+import functools
 
 from orjson import loads, JSONEncodeError
-from typing import Callable, Union
+from typing import Callable, Union, Dict, List
 
 from cryptography import x509
 from cryptography.x509.oid import NameOID, ExtensionOID, ExtendedKeyUsageOID
@@ -13,34 +15,25 @@ from cryptography.hazmat.primitives.asymmetric import padding
 
 from certsGenerator.helpers import loadFile
 
+
 checkRegistry = []
 
 
-class MetaRegistry(type):
-    @classmethod
-    def __prepare__(mcl, name, bases):  # type: ignore
-        def register_check(r) -> Callable:  # type: ignore
-            def deco(f: Callable) -> Callable:
-                checkRegistry.append(f)
-                return f
-
-            return deco
-
-        d = dict()
-        d["register_check"] = register_check
-        return d
-
-    def __new__(mcl, name, bases, dct):  # type: ignore
-        del dct["register_check"]
-        cls = super().__new__(mcl, name, bases, dct)
-        return cls
-
-
-class Conf(object, metaclass=MetaRegistry):
+class Conf:
     def __init__(self, confFile: str):
         self.general = self._load(confFile)
-        self._checkConf()
         self.fileExtenstions = self._getFileExt()
+        self.checkRegistry = checkRegistry
+
+        # run checks
+        self._checkCertName()
+        self._checkEncoding()
+        self._checkExtendedKeyUsage()
+        self._checkKeyUsage()
+        self._checkNotValidDate()
+        self._checkPubKeyEncodingPresent()
+        self._checkRedundantNames()
+        self._checkSerialization()
 
     def getCert(self, certName: str) -> dict:
         certs = self.general["certs"]
@@ -50,7 +43,9 @@ class Conf(object, metaclass=MetaRegistry):
                 conf = cert
                 break
         if not (len(conf) > 0):
-            raise ValueError(f'cert name "{certName}" not found')
+            logging.error(f"cert {certName} not found")
+            raise ValueError()
+            sys.exit()
         return conf["conf"]
 
     def get(self, certName: str, field: str, isExt: bool = True) -> dict:
@@ -60,7 +55,8 @@ class Conf(object, metaclass=MetaRegistry):
         elif certConf[field]:
             return certConf[field]
         else:
-            raise ValueError(f"{field} not found in conf")
+            logging.error(f"{field} not found in conf")
+            raise ValueError()
             sys.exit()
 
     def getCertPath(self, certName: str, ext: str) -> str:
@@ -86,7 +82,8 @@ class Conf(object, metaclass=MetaRegistry):
             passphrase = loadFile(passFile)
 
             if not type(passphrase) == bytes:
-                raise ValueError("passphrase should be of bytes type")
+                logging.error("passphrase should be of bytes type")
+                raise ValueError()
                 sys.exit()
             return passphrase
         else:
@@ -102,20 +99,26 @@ class Conf(object, metaclass=MetaRegistry):
                 except JSONEncodeError as e:
                     sys.exit(f"enable to load string from {fileName}: {e}")
         except OSError as e:
-            print(
-                'please make sure to execute the script like this: "python src/main.py"'
-            )
-            sys.exit(f"failed to open {fileName}: {e}")
+            logging.error(f"failed to open {fileName}: {e}")
         return generalConf
 
     def _getFileExt(self) -> dict:
         return self.general["defaults"]["file_extentions"]
 
-    def _checkConf(self) -> None:
-        for fun in checkRegistry:
-            fun()
+    def _checkPubKeyEncodingPresent(self) -> None:
+        # check if the public key encoding is persent
+        certs = self.general["certs"]
+        for cert in certs:
+            try:
+                cert.get("conf").get("public_key").get("encoding")
+            except KeyError as e:
+                certName = cert.get("name")
+                logging.error(
+                    f"Configuration file error: can't find public key encoding for {certName}"
+                )
+                raise ValueError()
+                sys.exit()
 
-    @register_check  # type: ignore
     def _checkRedundantNames(self) -> None:
         # check if no redundant certs names
         certs = self.general["certs"]
@@ -127,10 +130,10 @@ class Conf(object, metaclass=MetaRegistry):
                 names[cert["name"]] = 1
         for k, v in names.items():
             if v > 1:
-                raise ValueError(f"Configuration file error: {k} appears {v} times")
+                logging.error(f"Configuration file error: {k} appears {v} times")
+                raise ValueError()
                 sys.exit()
 
-    @register_check  # type: ignore
     def _checkNotValidDate(self) -> None:
         # check not_valid_before not_valid_after
         certs = self.general["certs"]
@@ -145,37 +148,77 @@ class Conf(object, metaclass=MetaRegistry):
                     days=certConf["not_valid_before"]
                 )
             else:
-                raise ValueError(f'invalid value from {nvb}, should be of int or "now"')
+                logging.error(f'invalid value from {nvb}, should be of int or "now"')
+                raise ValueError()
                 sys.exit()
 
-    @register_check  # type: ignore
     def _checkCertName(self) -> None:
         certs = self.general["certs"]
         for cert in certs:
             subjectName = cert["conf"]["subject_name"]
             certName = cert["name"]
             if certName != subjectName:
-                raise ValueError(
+                logging.error(
                     f"certname {certName} has to be the same than the subject name, found {subjectName}"
                 )
+                raise ValueError()
                 sys.exit()
 
-    @register_check  # type: ignore
     def _checkKeyUsage(self) -> None:
         certs = self.general["certs"]
         for cert in certs:
-            for ku in cert["extensions"]["KeyUsage"]["items"]:
-                if ku.upper not in self.keyUsage:
-                    raise ValueError(f"{ku} not found in allowed keyUsage")
-                sys.exit()
+            certName = cert.get("name")
+            try:
+                for ku in cert["conf"]["extensions"]["KeyUsage"]["items"]:
+                    if ku.lower() not in self.keyUsage:
+                        logging.error(
+                            f"{ku} not found in allowed keyUsage for {certName}"
+                        )
+                        raise ValueError()
+                        sys.exit()
+            except Exception:
+                logging.error(f"can't find KeyUsages params for {certName}")
 
-    @register_check  # type: ignore
     def _checkExtendedKeyUsage(self) -> None:
         certs = self.general["certs"]
         for cert in certs:
-            for ku in cert["extensions"]["ExtendedKeyUsage"]["items"]:
-                if ku.upper() not in self.extendedKeyUsageMapping.keys():
-                    raise ValueError(f"{ku} not found in allowed ExtendedKeyUsage")
+            if cert.get("conf").get("extensions").get("ExtendedKeyUsage"):
+                for ku in (
+                    cert.get("conf")
+                    .get("extensions")
+                    .get("ExtendedKeyUsage")
+                    .get("items")
+                ):
+                    if ku.upper() not in self.extendedKeyUsageMapping.keys():
+                        logging.error(f"{ku} not found in allowed ExtendedKeyUsage")
+                        raise ValueError()
+                        sys.exit()
+
+    def _checkEncoding(self) -> None:
+        certs = self.general["certs"]
+        for cert in certs:
+            certName = cert.get("name")
+            enc_priv = cert.get("conf").get("private_key").get("encoding").upper()
+            enc_pub = cert.get("conf").get("public_key").get("encoding").upper()
+            if (enc_priv not in self.encodingMapping.keys()) or (
+                enc_pub not in self.encodingMapping.keys()
+            ):
+                logging.error(
+                    f"encoding not found for {certName} in allowed encoding formats"
+                )
+                raise ValueError()
+                sys.exit()
+
+    def _checkSerialization(self) -> None:
+        certs = self.general["certs"]
+        for cert in certs:
+            certName = cert.get("name")
+            serialization = cert.get("conf").get("private_key").get("serialization")
+            if serialization not in self.serializationMapping.keys():
+                logging.error(
+                    f"serialization not found for private key of {certName} in allowed serialization formats"
+                )
+                raise ValueError()
                 sys.exit()
 
     nameAttributesMapping = {
@@ -186,7 +229,7 @@ class Conf(object, metaclass=MetaRegistry):
         "COMMON_NAME": NameOID.COMMON_NAME,
     }
 
-    extentionMapping = {
+    extensionMapping = {
         "SubjectAlternativeName": x509.SubjectAlternativeName,
         "KeyUsage": x509.KeyUsage,
         "BasicConstraints": x509.BasicConstraints,
@@ -197,11 +240,15 @@ class Conf(object, metaclass=MetaRegistry):
         "IPAddress": x509.IPAddress,
     }
 
-    # one supported for the moment
+    # one supported at the moment
     curveMapping = {"SECP521R1": ec.SECP521R1()}
 
-    serializationMapping = {
+    encodingMapping = {
         "PEM": serialization.Encoding.PEM,
+        "DER": serialization.Encoding.DER,
+    }
+
+    serializationMapping = {
         "PKCS8": serialization.PrivateFormat.PKCS8,
         "TraditionalOpenSSL": serialization.PrivateFormat.TraditionalOpenSSL,
         "Raw": serialization.PrivateFormat.Raw,
@@ -216,19 +263,17 @@ class Conf(object, metaclass=MetaRegistry):
 
     hashMapping = {"sha512": hashes.SHA512(), "sha256": hashes.SHA256()}
 
-    keyUsage = set(
-        [
-            "digital_signature",
-            "content_commitment",
-            "key_encipherment",
-            "data_encipherment",
-            "key_agreement",
-            "key_cert_sign",
-            "crl_sign",
-            "encipher_only",
-            "decipher_only",
-        ]
-    )
+    keyUsage = [
+        "digital_signature",
+        "content_commitment",
+        "key_encipherment",
+        "data_encipherment",
+        "key_agreement",
+        "key_cert_sign",
+        "crl_sign",
+        "encipher_only",
+        "decipher_only",
+    ]
 
     extendedKeyUsageMapping = {
         "SERVER_AUTH": x509.oid.ExtendedKeyUsageOID.SERVER_AUTH,
